/*
 *            Vuze Vanish Backend GPL Version 2 License
 * 
 * The distributed software (Software) is named the "Vuze Vanish Backend" and
 * serves as a backend storage system for the Vanish system (described at
 * http://vanish.cs.washington.edu/). The Software consists of Vuze (a.k.a.
 * Azureus) Bittorrent client, version  4.2.0.2 (original and
 * latest source code of Vuze is available at http://azureus.sourceforge.net).
 * 
 * Copyright 2009, University of Washington (C) 
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * All bug reports and technical questions shall be sent to:
 * vanish @ cs.washington.edu.
 * 
 */

package edu.washington.cs.vanish.internal.backend.impl;

import edu.washington.cs.vanish.internal.backend.TooManySharesLostVanishException;
import edu.washington.cs.vanish.internal.backend.VanishBackendException;
import edu.washington.cs.vanish.internal.backend.rpcimpl.RPCVanishBackendImpl;
import edu.washington.cs.vanish.logging.VanishLogger;
/**
 * 
 * @author roxana
 *
 */
public class VuzeRPCBackendHandler {
	protected static VuzeVanishBackendImpl backend;
	protected static VanishLogger LOG;

	public byte[][] getShares(Object[] locations, int threshold)
	throws VanishBackendException, TooManySharesLostVanishException {
		if (! isInitialized()) {
			throw new VanishBackendException("Not initialized");
		}
		byte[][] locs = RPCVanishBackendImpl.fromByteArrayToByteArray(
				locations);
		return backend.getShares(locs, threshold);
	}

	public int pushShares(Object[] shares, Object[] locations)
	throws VanishBackendException {
		if (! isInitialized()) {
			throw new VanishBackendException("Not initialized");
		}
		backend.pushShares(
				RPCVanishBackendImpl.fromByteArrayToByteArray(shares),
				RPCVanishBackendImpl.fromByteArrayToByteArray(locations));
		return 0;  // just for XMLRPC, whose return can't be void.
	}

	public boolean isInitialized() {
		return backend != null && backend.isInitialized();
	}
	
	protected static void init(VuzeVanishBackendImpl _backend,
			                   VanishLogger _log) {
		backend = _backend;
		LOG = _log;
	}
}
