/*
 *            Vuze Vanish Backend GPL Version 2 License
 * 
 * The distributed software (Software) is named the "Vuze Vanish Backend" and
 * serves as a backend storage system for the Vanish system (described at
 * http://vanish.cs.washington.edu/). The Software consists of Vuze (a.k.a.
 * Azureus) Bittorrent client, version  4.2.0.2 (original and
 * latest source code of Vuze is available at http://azureus.sourceforge.net).
 * 
 * Copyright 2009, University of Washington (C) 
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * All bug reports and technical questions shall be sent to:
 * vanish @ cs.washington.edu.
 * 
 */

package edu.washington.cs.vanish.internal.backend.impl;

import java.io.IOException;
import java.net.InetAddress;
import java.net.UnknownHostException;

import org.apache.commons.configuration.ConfigurationException;
import org.apache.xmlrpc.XmlRpcException;
import org.apache.xmlrpc.server.PropertyHandlerMapping;
import org.apache.xmlrpc.server.XmlRpcServer;
import org.apache.xmlrpc.server.XmlRpcServerConfigImpl;
import org.apache.xmlrpc.webserver.WebServer;

import edu.washington.cs.vanish.conf.VanishConfiguration;
import edu.washington.cs.vanish.internal.InvalidArgumentVanishException;
import edu.washington.cs.vanish.internal.VanishException;
import edu.washington.cs.vanish.internal.backend.rpcimpl.RPCVanishBackendDefaults;
import edu.washington.cs.vanish.logging.LogStat;
import edu.washington.cs.vanish.logging.VanishLogger;
import edu.washington.cs.vanish.util.Initializable;

/**
 * TODO(roxana): This class, along with the other RPC-Vuze ones were added
 * at the last minute and are HACKY.
 * 
 * @author roxana
 *
 */

public class VuzeRPCBackendImpl implements Initializable {
	private final VanishLogger LOG;
	private final VanishConfiguration CONF;

	private final WebServer web_server;
	
	private final VuzeVanishBackendImpl backend;
	
	private boolean is_initialized = false;
	
	public VuzeRPCBackendImpl(VanishConfiguration conf, VanishLogger log)
	throws InvalidArgumentVanishException {
		this.LOG = log;
		this.CONF = conf;
		
		int kPort = this.CONF.getInt(
				VuzeBackendDefaults.CONF_VUZE_VANISH_RPCSERVER_PORT,
				VuzeBackendDefaults.CONF_VUZE_VANISH_RPCSERVER_PORT_DEFAULT);
		
		WebServer ws = null;
		try {
			ws = new WebServer(kPort,
					InetAddress.getByAddress(new byte[]{ 127, 0, 0, 1 }));
		} catch (UnknownHostException e) {  // Can't happen.
			e.printStackTrace(); 
		}
		
		web_server = ws;
		
		backend = new VuzeVanishBackendImpl(conf, log);
	}
	
    // @Override
	public void init() throws VanishException {
		final LogStat logstat = LOG.logStat("VuzeRPCBackendImpl.init");
		assert(! isInitialized());
		VanishException exception = null;
		try {
            // Then the server:
			configureXmlRpcServer();
            
			// Start the RPC server -- should be the last thing we do in init.
			try { web_server.start(); }
			catch (IOException e) {
				throw new VanishException("Failed to init", e);
			}
			
			
			// First the backend.
			backend.init();
			
			// Other initializations go here.
			// ...
			
			// Last line must be:
			VuzeRPCBackendHandler.init(backend, LOG);
		} catch (VanishException e) {
			exception = e;
			throw e;
		} finally {
			logstat.end(exception);
		}
	    is_initialized = true;  // must be last instruction!
	}
	
	// TODO(roxana): rm horrible copy/paste from VanishXMLRPCServer. 
	private void configureXmlRpcServer()
	throws VanishException {
		XmlRpcServer xmlRpcServer = web_server.getXmlRpcServer();
		xmlRpcServer.setMaxThreads(5);  // TODO(roxana): Config.
	    
	    // Add the handlers.
	    PropertyHandlerMapping phm = new PropertyHandlerMapping();
		try {
			phm.addHandler(
			 RPCVanishBackendDefaults.CONF_VANISH_BACKEND_SERVICE_NAME_DEFAULT,
			 VuzeRPCBackendHandler.class);
		} catch (XmlRpcException e) {
			throw new VanishException(e.getMessage());
		}
	    xmlRpcServer.setHandlerMapping(phm);
	    
	    // Configure a bit more.
	    XmlRpcServerConfigImpl serverConfig =
	    	(XmlRpcServerConfigImpl)xmlRpcServer.getConfig();
	    serverConfig.setEnabledForExtensions(true);
	    serverConfig.setEnabledForExceptions(true);
	    serverConfig.setContentLengthOptional(false);
	}
	
	// @Override
	public void stop() throws VanishException {
		assert(isInitialized());
		web_server.shutdown();  // first the server
		backend.stop();  // then the backend
	}
	
	// @Override
	public boolean isInitialized() { return is_initialized; }

	// Helper functions:
	
	protected static void doWork(VanishConfiguration conf)
	throws VanishException {
		VanishLogger log = VanishLogger.getLogger("vanish.rpcbackend");
		
		VuzeRPCBackendImpl backend_server = new VuzeRPCBackendImpl(conf, log);
		
		backend_server.init();
		backend_server.serve();
		backend_server.stop();
	}
	
	// TODO(roxana): Do other things here: configuration refreshing, etc.
	private void serve() {
		assert(isInitialized());
		
		while (true) {
			try { Thread.sleep(3600000L); }
			catch (InterruptedException e) { e.printStackTrace(); }
		}
	}	
	
	public static void main(String args[]) {
		VanishConfiguration config = null;
		try {
			config = new VanishConfiguration(
					VuzeBackendDefaults.VUZE_BACKEND_CONFIGURATION_FILE);
		} catch (ConfigurationException e) {
			e.printStackTrace();
			System.exit(1);
		}
		try {
			doWork(config);
		} catch (VanishException e) {
			e.printStackTrace();
			System.exit(1);
		}
	}
}
